package com.smcic.core.advice;

import lombok.extern.slf4j.Slf4j;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestControllerAdvice
@Slf4j
public class APIResponseAdvice implements ResponseBodyAdvice<Object> {

    //自动处理APINoDataResponse，包装为APINoDataResponse
    @ExceptionHandler(APIException.class)
    public APINoDataResponse handleApiException(HttpServletRequest request, APIException ex, HttpServletResponse response) {
        log.error("process url {} failed", request.getRequestURL().toString(), ex);
        APINoDataResponse apiNoDataResponse = new APINoDataResponse();
        apiNoDataResponse.setCode(ex.getErrorCode());
        apiNoDataResponse.setMessage(ex.getErrorMessage());

        response.setStatus(ex.getErrorCode() == -1 ? HttpStatus.INTERNAL_SERVER_ERROR.value() : HttpStatus.OK.value());
        return apiNoDataResponse;
    }

    @ExceptionHandler(BizException.class)
    public APIResponse handleBizException(HttpServletRequest request, BizException ex, HttpServletResponse response) {
        log.warn("error", ex);
        APIResponse apiNoDataResponse = new APIResponse();
        apiNoDataResponse.setCode(ex.getErrorCode());
        apiNoDataResponse.setMessage(ex.getErrorMessage());
        return apiNoDataResponse;
    }

    /**
     * 拦截未知的运行时异常
     *
     * @author fengshuonan
     * @date 2020/12/16 15:12
     */
    @ExceptionHandler(Throwable.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public APINoDataResponse serverError(Throwable ex) {
        log.error("服务器运行异常", ex);
        APINoDataResponse apiResponse = new APINoDataResponse();
        apiResponse.setCode(-1);
        apiResponse.setMessage("服务器运行异常");
        return apiResponse;
    }

    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public APINoDataResponse validationBodyException(MethodArgumentNotValidException exception) {

        BindingResult result = exception.getBindingResult();
        if (result.hasErrors()) {

            List<ObjectError> errors = result.getAllErrors();

            errors.forEach(p -> {

                FieldError fieldError = (FieldError) p;
                log.error("Data check failure : object{" + fieldError.getObjectName() + "},field{" + fieldError.getField() +
                        "},errorMessage{" + fieldError.getDefaultMessage() + "}");

            });

        }
        APINoDataResponse apiResponse = new APINoDataResponse();
        apiResponse.setCode(-1);
        apiResponse.setMessage("请填写正确信息");
        return apiResponse;
    }


    //仅当方法或类没有标记@NoAPIResponse才自动包装
    @Override
    public boolean supports(MethodParameter returnType, Class converterType) {
        return returnType.getParameterType() != APIResponse.class && returnType.getParameterType() != APINoDataResponse.class
                && AnnotationUtils.findAnnotation(returnType.getDeclaringClass(), NoAPIResponse.class) == null;
    }

    //自动包装外层APIResposne响应
    @Override
    public Object beforeBodyWrite(Object body,
                                  MethodParameter returnType,
                                  MediaType selectedContentType,
                                  Class<? extends HttpMessageConverter<?>> selectedConverterType,
                                  ServerHttpRequest request, ServerHttpResponse response) {


        ResponseMsg responseMsg = returnType.getMethodAnnotation(ResponseMsg.class);
        String msg = "OK";

        if (null != responseMsg) {
            msg = responseMsg.value();
        }

        if (null == body) {
            APINoDataResponse apiResponse = new APINoDataResponse();
            apiResponse.setCode(0);

            apiResponse.setMessage(msg);
            return apiResponse;
        } else {
            APIResponse apiResponse = new APIResponse();

            apiResponse.setCode(0);
            apiResponse.setMessage(msg);
            apiResponse.setData(body);
            return apiResponse;
        }

    }

}
